function B = PHL(sw_path,XC,X_Target,X_Query,Mask,c_num,lambda,stop,MaxIter,phl_model_path,eps)
%% step1: Pre computing
if exist(sw_path,'file')
    m = load(sw_path,'XC','Rw','Hb_Rw_inv','Rw_invt_Sb','Hb','');
    Rw = m.Rw; 
    Hb_Rw_inv = m.Hb_Rw_inv;    Rw_invt_Sb = m.Rw_invt_Sb;
    Hb = m.Hb;
    XC = m.XC;
else
    index_pos = Mask==1;
    index_neg = Mask==0;
    X_Diff = X_Target - X_Query;
    Hw = X_Diff(:,index_pos)' * XC;
    Hb = X_Diff(:,index_neg)' * XC;
    Sw = Hw' * Hw;
    Sb = Hb' * Hb;

    dim = size(Sw,1);
    [u s v] = svd(Sw);
    Rw = (u*sqrt(s+0.1*eye(dim)))';
    Hb_Rw_inv = Hb/Rw;
    Rw_invt_Sb = ((Sb') / Rw)';  
    save(sw_path,'XC','Rw','Hw','Hb','Sw','Sb','Hb_Rw_inv','Rw_invt_Sb','X_Target','X_Query','Mask');
end
W = [Hb; sqrt(lambda)*Rw];
%% Step 2: Cholesky Decomposition
p = size(XC,2); q = c_num;
z0 = zeros(p,1,'single');
Z0 = repmat(z0,1,q);
A = ones(p,q,'single');
B = ones(p,q,'single');


%% Step 3: Repeat minimization
% eps = 0.01;
for iter = 1:MaxIter
    A0 = A; B0 = B;
    % 3.1:  solve q lasso problem
    % min  0.5*x'*A*x+b'*x+\lambda*|x|
    % [x]=L1QP_FeatureSign_yang(lambda,A,b)
    MA = [Hb_Rw_inv*A; Z0];
    if iter == 1
        y = MA(:,1);
        tmp = lars(W,y,'lassor',stop);     tmp = tmp';
        beta_j = tmp(:,end);
        for j=1:q
            B(:,j) = beta_j;
        end
        fprintf('Iter: %d, Lar %d / %d , nonzeros: %d\n',iter,j,q,length(find(beta_j~=0)));
    else
        parfor j = 1:q
            y = MA(:,j);%[MA(:,j); z0];
            tmp = lars(W,y,'lassor',stop);     tmp = tmp';
            beta_j = tmp(:,end);

            B(:,j) = beta_j;
            fprintf('Iter: %d, Lar %d / %d , nonzeros: %d\n',iter,j,q,length(find(beta_j~=0)));
        end
    end
    %3.2 SVD
    [U D V] =  svd( Rw_invt_Sb * B );
    A = U(:,1:q)*V';
    
    da = (A-A0).^2; db = (B-B0).^2;
    err_a = sum(da(:)) / (p*q);
    err_b = sum(db(:)) / (p*q);
    if err_b < eps
        break;
    end
    fprintf('Iter %d: A change:%f B change:%f\n',iter,err_a,err_b);
end
fprintf('Iter %d: A change:%f B change:%f\n',iter,err_a,err_b);
save(phl_model_path,'XC','B');
